#pragma once
/*
 * Interval
 * Copyright (C) 2014, 2016, 2018, 2019, 2023, 2025  Pavel Brychta http://www.xpablo.cz
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the MIT License
 */
#include <Arduino.h>

class Interval {
  public:
    // Enumeration for interval modes
    enum IntervalMode {
        COMPATIBILITY = 0,
        ONESHOT = 1,
        PERIODIC = 2
    };

  protected:
    uint32_t _timefrom = 0; // Start time of the interval
    uint32_t _timeout = 0; // Timeout duration in milliseconds
    IntervalMode _mode = COMPATIBILITY; // Current mode of operation
    bool _active = true; // Active flag for oneshot and compatibilitymode
  public:
    Interval() = default;

    // Constructor for periodic mode with autostart
    explicit Interval(const uint32_t tmout)
        : _timefrom(millis()), _timeout(tmout), _mode(PERIODIC) {
    }

    // Checks if the interval has expired based on the current mode
    bool expired();

    // Sets the interval in compatibility mode
    void set(uint32_t tmout);

    // Sets the interval in oneshot mode
    void setOneshot(uint32_t tmout);

    // Sets the interval in periodic mode
    void setPeriodic(uint32_t tmout);

    // Reloads the interval timer without changing mode or timeout
    void reload();

    // Returns the elapsed time since the interval started
    [[nodiscard]] uint32_t elapsed() const;

    // Returns the remaining time until the interval expires
    [[nodiscard]] uint32_t remains() const;
};
